Skip to main content

squawk npm

Linter for Postgres migrations & SQL

Quick Start | Playground | Rules Documentation | GitHub Action | DIY GitHub Integration

Why?

Prevent unexpected downtime caused by database migrations and encourage best practices around Postgres schemas and SQL.

Install

npm install -g squawk-cli

# or via PYPI
pip install squawk-cli

# or install binaries directly via the releases page
https://github.com/sbdchd/squawk/releases

Or via Docker

You can also run Squawk using Docker. The official image is available on GitHub Container Registry.

# Assuming you want to check sql files in the current directory
docker run --rm -v $(pwd):/data ghcr.io/sbdchd/squawk:latest *.sql

Or via the Playground

Use the WASM powered playground to check your SQL locally in the browser!

https://play.squawkhq.com

Or via VSCode

https://marketplace.visualstudio.com/items?itemName=sbdchd.squawk

Usage

❯ squawk example.sql
warning[prefer-bigint-over-int]: Using 32-bit integer fields can result in hitting the max `int` limit.
  ╭▸ example.sql:6:10
  │
6 │     "id" serial NOT NULL PRIMARY KEY,
  │          ━━━━━━
  │
  ├ help: Use 64-bit integer values instead to prevent hitting this limit.
  ╭╴
6 │     "id" bigserial NOT NULL PRIMARY KEY,
  ╰╴         +++
warning[prefer-identity]: Serial types make schema, dependency, and permission management difficult.
  ╭▸ example.sql:6:10
  │
6 │     "id" serial NOT NULL PRIMARY KEY,
  │          ━━━━━━
  │
  ├ help: Use an `IDENTITY` column instead.
  ╭╴
6 -     "id" serial NOT NULL PRIMARY KEY,
6 +     "id" integer generated by default as identity NOT NULL PRIMARY KEY,
  ╰╴
warning[prefer-text-field]: Changing the size of a `varchar` field requires an `ACCESS EXCLUSIVE` lock, that will prevent all reads and writes to the table.
  ╭▸ example.sql:7:13
  │
7 │     "alpha" varchar(100) NOT NULL
  │             ━━━━━━━━━━━━
  │
  ├ help: Use a `TEXT` field with a `CHECK` constraint.
  ╭╴
7 -     "alpha" varchar(100) NOT NULL
7 +     "alpha" text NOT NULL
  ╰╴
warning[require-concurrent-index-creation]: During normal index creation, table updates are blocked, but reads are still allowed.
   ╭▸ example.sql:10:1
   │
10 │ CREATE INDEX "field_name_idx" ON "table_name" ("field_name");
   │ ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━
   │
   ├ help: Use `concurrently` to avoid blocking writes.
   ╭╴
10 │ CREATE INDEX concurrently "field_name_idx" ON "table_name" ("field_name");
   ╰╴             ++++++++++++
warning[constraint-missing-not-valid]: By default new constraints require a table scan and block writes to the table while that scan occurs.
   ╭▸ example.sql:12:24
   │
12 │ ALTER TABLE table_name ADD CONSTRAINT field_name_constraint UNIQUE (field_name);
   │                        ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━
   │
   ╰ help: Use `NOT VALID` with a later `VALIDATE CONSTRAINT` call.
warning[disallowed-unique-constraint]: Adding a `UNIQUE` constraint requires an `ACCESS EXCLUSIVE` lock which blocks reads and writes to the table while the index is built.
   ╭▸ example.sql:12:28
   │
12 │ ALTER TABLE table_name ADD CONSTRAINT field_name_constraint UNIQUE (field_name);
   │                            ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━
   │
   ╰ help: Create an index `CONCURRENTLY` and create the constraint using the index.

Find detailed examples and solutions for each rule at https://squawkhq.com/docs/rules
Found 6 issues in 1 file (checked 1 source file)

squawk --help

Find problems in your SQL

Usage: squawk [OPTIONS] [path]... [COMMAND]

Commands:
  server            Run the language server
  upload-to-github  Comment on a PR with Squawk's results
  help              Print this message or the help of the given subcommand(s)

Arguments:
  [path]...
          Paths or patterns to search

Options:
      --exclude-path <EXCLUDED_PATH>
          Paths to exclude

          For example:

          `--exclude-path=005_user_ids.sql --exclude-path=009_account_emails.sql`

          `--exclude-path='*user_ids.sql'`

  -e, --exclude <rule>
          Exclude specific warnings

          For example: --exclude=require-concurrent-index-creation,ban-drop-database

  -i, --include <rule>
          Include opt-in rules that are disabled by default

          Rules listed in --exclude take precedence over --include.

          For example: --include=require-table-schema

      --pg-version <PG_VERSION>
          Specify postgres version

          For example: --pg-version=13.0

      --debug <format>
          Output debug format

          [possible values: lex, parse, ast]

      --reporter <REPORTER>
          Style of error reporting

          [possible values: tty, gcc, json, gitlab]

      --stdin-filepath <filepath>
          Path to use in reporting for stdin

      --verbose
          Enable debug logging output

  -c, --config <CONFIG_PATH>
          Path to the squawk config file (.squawk.toml)

      --assume-in-transaction
          Assume that a transaction will wrap each SQL file when run by a migration tool

          Use --no-assume-in-transaction to override any config file that sets this

      --no-error-on-unmatched-pattern
          Do not exit with an error when provided path patterns do not match any files

  -h, --help
          Print help (see a summary with '-h')

  -V, --version
          Print version

Rules

Individual rules can be disabled via the --exclude flag

squawk --exclude=adding-field-with-default,disallowed-unique-constraint example.sql

Disabling rules via comments

Rule violations can be ignored via the squawk-ignore comment:

-- squawk-ignore ban-drop-column
alter table t drop column c cascade;

You can also ignore multiple rules by making a comma seperated list:

-- squawk-ignore ban-drop-column, renaming-column,ban-drop-database
alter table t drop column c cascade;

To ignore a rule for the entire file, use squawk-ignore-file:

-- squawk-ignore-file ban-drop-column
alter table t drop column c cascade;
-- also ignored!
alter table t drop column d cascade;

Or leave off the rule names to ignore all rules for the file

-- squawk-ignore-file
alter table t drop column c cascade;
create table t (a int);

Configuration file

Rules can also be disabled with a configuration file.

By default, Squawk will traverse up from the current directory to find a .squawk.toml configuration file. You may specify a custom path with the -c or --config flag.

squawk --config=~/.squawk.toml example.sql

The --exclude flag will always be prioritized over the configuration file.

Example .squawk.toml

excluded_rules = [
    "require-concurrent-index-creation",
    "require-concurrent-index-deletion",
]

See the Squawk website for documentation on each rule with examples and reasoning.

Bot Setup

Squawk works as a CLI tool but can also create comments on GitHub Pull Requests using the upload-to-github subcommand.

Here's an example comment created by squawk using the example.sql in the repo:

https://github.com/sbdchd/squawk/pull/14#issuecomment-647009446

See the "GitHub Integration" docs for more information.

pre-commit hook

Integrate Squawk into Git workflow with pre-commit. Add the following to your project's .pre-commit-config.yaml:

repos:
  - repo: https://github.com/sbdchd/squawk
    rev: v2.64.0
    hooks:
      - id: squawk
        files: path/to/postgres/migrations/written/in/sql

Note the files parameter as it specifies the location of the files to be linted.

Prior Art / Related

Related Blog Posts / SE Posts / PG Docs

Dev

cargo install
cargo run
./s/test
./s/lint
./s/fmt

... or with nix:

$ nix develop
[nix-shell]$ cargo run
[nix-shell]$ cargo insta review
[nix-shell]$ ./s/test
[nix-shell]$ ./s/lint
[nix-shell]$ ./s/fmt

Adding a New Rule

When adding a new rule, running cargo xtask new-rule will create stubs for your rule in the Rust crate and in Documentation site.

cargo xtask new-rule 'prefer big serial'

Releasing a New Version

  1. Run s/update-version

    # update version in squawk/Cargo.toml, package.json, flake.nix to 4.5.3
    s/update-version 4.5.3
    
  2. Update the CHANGELOG.md

    Include a description of any fixes / additions. Make sure to include the PR numbers and credit the authors.

  3. Create a new release on GitHub

    Use the text and version from the CHANGELOG.md

Algolia

The squawkhq.com Algolia index can be found on the crawler website. Algolia reindexes the site every day at 5:30 (UTC).

How it Works

Squawk uses its parser (based on rust-analyzer's parser) to create a CST. The linters then use an AST layered on top of the CST to navigate and record warnings, which are then pretty printed!

Metadata

Release files for squawk-cli 2.64.0

For a detailed explanation of source distributions (sdists) and built distributions (wheels), please see the package formats documentation.

Source distribution (sdist)

Source distribution for squawk-cli 2.64.0
File Size Uploaded
squawk_cli-2.64.0.tar.gz 1.3 MB Details

Built distributions (wheels)

Table of built distributions (wheels) for squawk-cli 2.64.0
File
squawk_cli-2.64.0-py3-none-win_amd64.whl Python 3 none Windows x86-64 Details
squawk_cli-2.64.0-py3-none-win32.whl Python 3 none Windows x86-32 Details
squawk_cli-2.64.0-py3-none-manylinux_2_28_x86_64.whl Python 3 none Linux glibc 2.28+ x86-64 Details
squawk_cli-2.64.0-py3-none-manylinux_2_17_aarch64.manylinux2014_aarch64.whl Python 3 none Linux glibc 2.17+ ARM64 Details
squawk_cli-2.64.0-py3-none-macosx_11_0_arm64.whl Python 3 none macOS 11.0+ ARM64 Details
squawk_cli-2.64.0-py3-none-macosx_10_12_x86_64.whl Python 3 none macOS 10.12+ x86-64 Details

Total release size: 38.6 MB

Release files / squawk_cli-2.64.0.tar.gz

Download URL squawk_cli-2.64.0.tar.gz
Size 1.3 MB
Tags Source
SHA-256 checksum
How to use checksums
eae49360beb8280b6ed16c2fe2596e4d0d18db39992f36d47fab43d2c7415b74
BLAKE2b-256 checksum
How to use checksums
9eb1daabaa54c269c4ef2c2683156beedede01c993688042f00cae58190360cf
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release files / squawk_cli-2.64.0-py3-none-win_amd64.whl

Download URL squawk_cli-2.64.0-py3-none-win_amd64.whl
Size 5.6 MB
Tags Python 3 Windows x86-64
SHA-256 checksum
How to use checksums
53f3f88228cd511a6d6fb4bc8f7f5cc4d178eac80b46d6b3a4ad91e593a7a27c
BLAKE2b-256 checksum
How to use checksums
34900d694d881459e6d1b338148d502fad76829cc3bd833b1e5eb292316615a5
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release files / squawk_cli-2.64.0-py3-none-win32.whl

Download URL squawk_cli-2.64.0-py3-none-win32.whl
Size 4.9 MB
Tags Python 3 Windows x86-32
SHA-256 checksum
How to use checksums
dbf4df9a6dff71e8e8883c0dd7c4b8f160bec143f73452eeaf65e66b43442176
BLAKE2b-256 checksum
How to use checksums
876905dd176751b31901eee3a4b8be137cf3dd71e07f2ec60ea96fd056e4994b
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release files / squawk_cli-2.64.0-py3-none-manylinux_2_28_x86_64.whl

Download URL squawk_cli-2.64.0-py3-none-manylinux_2_28_x86_64.whl
Size 8.0 MB
Tags Linux glibc 2.28+ x86-64 Python 3
SHA-256 checksum
How to use checksums
2cd2a390fbf7dc2a9fd48897af99f1af287ded56085239b0f753729f4ee4f4f9
BLAKE2b-256 checksum
How to use checksums
3f32e2fbc64832ec8ae7da8af380c3b4605862005684773343df1ab1b70c9a6f
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release files / squawk_cli-2.64.0-py3-none-manylinux_2_17_aarch64.manylinux2014_aarch64.whl

Download URL squawk_cli-2.64.0-py3-none-manylinux_2_17_aarch64.manylinux2014_aarch64.whl
Size 7.1 MB
Tags Linux glibc 2.17+ ARM64 Python 3
SHA-256 checksum
How to use checksums
650fa927b1031971f3cb8a0498223c5966842b6a059e6d042d1eacb2e3fd8f69
BLAKE2b-256 checksum
How to use checksums
223e1b7ddd94bc6c88474642630a8e11b5d1a445fb40df9ff0dd5574d83f876b
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release files / squawk_cli-2.64.0-py3-none-macosx_11_0_arm64.whl

Download URL squawk_cli-2.64.0-py3-none-macosx_11_0_arm64.whl
Size 5.7 MB
Tags Python 3 macOS 11.0+ ARM64
SHA-256 checksum
How to use checksums
57a21607b7b5a6b1a7679365ac3fb79fa6860263511a57bed1791434dd3ba940
BLAKE2b-256 checksum
How to use checksums
b3c88988bfe625db0902ca870db353d63396140b446e7c8cecee414e8492d1d9
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release files / squawk_cli-2.64.0-py3-none-macosx_10_12_x86_64.whl

Download URL squawk_cli-2.64.0-py3-none-macosx_10_12_x86_64.whl
Size 6.0 MB
Tags Python 3 macOS 10.12+ x86-64
SHA-256 checksum
How to use checksums
ee8978c30202f191d901fcac21daaafc9d300f2e5cb71fddd8b2eef65d96e596
BLAKE2b-256 checksum
How to use checksums
e32ae843a3a9cb4f3f4e3668550409404e5be42b0a9980df3bbcc242a204510c
Upload date
Uploaded using Trusted Publishing?
What is trusted publishing?
Yes
Uploaded via maturin/1.15.0

Release history Release notifications | RSS feed

2.66.0

7 release files

2.65.0

7 release files

This release

2.64.0 This release

7 release files

2.63.0

7 release files

2.61.0

7 release files

2.60.0

7 release files

2.59.0

7 release files

2.58.0

7 release files

2.55.0

7 release files

2.54.0

7 release files

2.53.0

7 release files

2.52.1

7 release files

2.52.0

7 release files

2.49.0

7 release files

2.48.0

7 release files

2.47.0

7 release files

2.44.0

7 release files

2.43.0

7 release files

2.42.0

7 release files

2.41.0

7 release files

2.40.1

7 release files

2.39.0

7 release files

2.38.0

7 release files

2.37.0

7 release files

2.34.0

7 release files

2.33.2

7 release files

2.33.1

7 release files

2.33.0

7 release files

2.31.0

7 release files

2.30.0

7 release files

2.29.0

7 release files

2.27.0

7 release files

2.26.0

7 release files

2.24.0

7 release files

2.23.0

7 release files

2.21.1

7 release files

2.21.0

7 release files

2.20.0

7 release files

2.19.0

7 release files

2.16.0

7 release files

2.15.0

7 release files

2.14.0

7 release files

2.13.0

7 release files

2.12.0

7 release files

2.11.0

7 release files

2.10.0

7 release files

2.9.0

7 release files

2.8.0

7 release files

2.7.0

7 release files

2.6.0

7 release files

2.5.0

7 release files

2.4.0

7 release files

2.3.0

7 release files

2.2.0

7 release files

2.1.0

7 release files

2.0.0

7 release files

1.6.1

7 release files

1.6.0

7 release files

1.5.5

7 release files

1.5.4

7 release files

1.4.0

7 release files

1.2.0

7 release files

1.1.0

3 release files

1.0.0

3 release files

0.28.0

3 release files

0.27.0

3 release files

0.26.0

3 release files

0.24.1

3 release files

0.24.0

3 release files

0.23.0

3 release files

0.22.0

3 release files

0.21.0

3 release files

Anthropic, PBC Visionary sponsor Bloomberg Visionary sponsor Hudson River Trading Visionary sponsor Meta Visionary sponsor NVIDIA Visionary sponsor Microsoft Sustainability sponsor Depot Continuous Integration AWS Cloud computing and Security Sponsor Datadog Monitoring Fastly CDN Google Download Analytics Sentry Error logging StatusPage Status page